/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2017 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#include "fesslix.h"

#include <iostream>


void cdf_example()
{
  // --------------------------------------------------------------------------
  // This example demonstrates how you can evaluate a CDF (Gumbel distribution)
  // and its inverse.
  // --------------------------------------------------------------------------
  tdouble& c1 = *FlxEngine->dataBox.ConstantBox.get("c1",true);
  tdouble& c2 = *FlxEngine->dataBox.ConstantBox.get("c2",true);
  c1 = 0.5;
  FlxFunction* your_fun = FlxEngine->dataBox.ReadManager.parse_function("cdf_inv(c1,gamma,mu=5,sd=2)");
  c2 = your_fun->calc();
  FlxFunction* your_fun2 = FlxEngine->dataBox.ReadManager.parse_function("cdf(c2,gamma,mu=5,sd=2)");
  // evaluate the functions on interval [0;1] and write to standard output
    for (tdouble d=0.05;d<ONE;d+=0.1) {
      c1 = d;
      c2 = your_fun->calc();
      c1 = your_fun2->calc();
      std::cout << "  " << c1 << "\t" << c2 << std::endl;
    }
  delete your_fun;
  delete your_fun2;
}

void rv_def()
{
  // --------------------------------------------------------------------------
  // This example shows how to generate a set of random variables using the 
  // fesslix-command language.
  // --------------------------------------------------------------------------
  std::string cms = "loadlib \"RND\";\n";
  cms += "const lambda = 1.0;\n";
  cms += "const epsi = 0.2;\n";
  cms += "const mode = exp(lambda-0.3^2)+epsi;\n";
  cms += "rbrv_set RBSET2 () {\n";
  cms += "  logn(t1,mode=mode,sd=sqrt((exp(0.3^2)-1)*exp(2*lambda+0.3^2)),epsilon=epsi)\n";
  cms += "};";
  flx_parse_file(cms,false,true,true);
}

void mci_simu()
{
  // --------------------------------------------------------------------------
  // A monte carlo simulation is performed, using the set of random variables 
  // defined in c++-function 'rv_def' and using a limit-state function defined
  // directly in the fesslix-command language.
  // --------------------------------------------------------------------------
  std::cout << "Monte Carlo simulation:" << std::endl;
  std::string cms = "var lsf = rbrv(RBSET2::t1) - mode;";
  cms += "mci(mciv;1e4;lsf) { rbrvsets=RBSET2; };";
  flx_parse_file(cms,false,true,true);
  std::cout << " -> result: " << exp(*(FlxEngine->dataBox.ConstantBox.get("mciv"))) << std::endl;
}


// this function is used as limit-state function in c++-function 'mci_simu'
const tdouble your_lsf(const tdouble* const rv, const tuint Ndim) {
  return rv[0]-1.;
}

void sus_simu()
{
  // --------------------------------------------------------------------------
  // Subset simulation is performed, using the set of random variables 
  // defined in c++-function 'rv_def' and using a limit-state function defined
  // in c++-function 'your_lsf'
  // --------------------------------------------------------------------------
  // subset simulation (with call-back limit-state)
  std::cout << "Subset simulation:" << std::endl;
  GlobalVar.logLevel = 4;
  FlxEngine->dataBox.FunBox.insert("myfun", new FunReadFunLSF_callback(your_lsf,"myfun",false) );
  std::string cms = "sus(100;10;myfun(RBSET2)) { rbrvsets=RBSET2; };";
  flx_parse_file(cms,false,true,true);
  std::cout << " -> result: " << exp(*(FlxEngine->dataBox.ConstantBox.get("sus_pr"))) << std::endl;
}


